*     JLdL 22Nov14.
*     
*     Add up the center series for the standard square wave and
*     _ its Fourier Conjugate, for plotting purposes.
*     
*     Note: this program is meant to be called from within the
*     _ shell script plot_sqrw_and_fc.plot.
*     
*     Parameters:
*     _ mpnts: a fixed parameter for the variable npnts.
*     
*     Variables:
*     _ iflag: a flag for the sub-interval to plot (input);
*     _ ermax: the maximum error acceptable in the result (input);
*     _ onepi: the value of \pi;
*     _ snorm: the overall normalization constant of the series;
*     _ mktrm: the maximum number of terms to add up on the series;
*     _ npnts: the number of points in the x half interval;
*     _ delta: the increment of x within its interval;
*     _ nptmn: the left end of the sub-interval;
*     _ nptmx: the right end of the sub-interval;
*     _ ixvar: the integer x variable (loop counter);
*     _ rxvar: the current real value of x;
*     _ xnorm: the x-dependent normalization constant;
*     _ ereff: the renormalized maximum acceptable error;
*     _ acsfn: the accumulator for the series of the function;
*     _ acsfc: the accumulator for the series of the conjugate;
*     _ iexit: the series loop exit flag;
*     _ ijtrm: the counter for the series terms (loop counter);
*     _ iktrm: the value of k for the series terms;
*     _ coef:  temporary for the coefficients of the series;
*     _ trmf:  temporary for the terms of the series for the function;
*     _ trmc:  temporary for the terms of the series for the conjugate;
*     _ tmax:  temporary for the larger of the variables trmf and trmc;
*     _ error: the current value of the error;
*     _ fnvar: the value of the function;
*     _ fcvar: the value of the conjugate.
*     
      PROGRAM PLOT_SQRW_AND_FC
*     
*     Include global integer parameters;
*     _ this file defines mpnts.
      include 'global_params.f'
*     
*     Double precision everything.
      implicit double precision(a-h,o-z)
*     
*     Set the interval flag; read from stdin in filter mode.
      read(*,*) iflag
*     
*     Set the maximum acceptable error; read from stdin in filter mode.
      read(*,*) ermax
*     
*     Define constants.
      onepi=dacos(-1.0d0)
*     
*     Define the global normalization for the series.
      snorm=2.0d0/onepi
*     
*     Set the maximum number of series terms to add up.
      mktrm=100000
*     
*     Set the number of data points.
      npnts=mpnts
*     
*     Define the point spacing along x, and the range of x.
      delta=onepi/dble(npnts)
*     
*     Define the x interval to run.
      if (iflag.eq.1) then
         nptmn=1
         nptmx=npnts-1
      else if (iflag.eq.-1) then
         nptmn=-npnts+1
         nptmx=-1
      else
         write(*,*) "wrong value of iflag: ",iflag
         stop
      end if
*     
*     Loop over the x interval.
      do ixvar=nptmn,nptmx
*     
*     Define the current value of x.
         rxvar=dble(ixvar)*delta
*     
*     Define the x-dependent normalization.
         xnorm=snorm/dsin(rxvar)
*     
*     Renormalize the maximum acceptable error accordingly.
         ereff=ermax/dabs(xnorm)
*     
*     Initialize the series accumulator for the function.
         acsfn=0.0d0
*     
*     Initialize the series accumulator for the conjugate.
         acsfc=0.0d0
*     
*     Initialize the exit flag.
         iexit=0
*     
*     Initialize a loop counter for the terms.
         ijtrm=0
         iktrm=2*ijtrm+1
*     
*     Add up the series; loop until the value of the error decays below
*     _ the precision threshold; this has to remain so for 10 terms, in
*     _ order to filter out accidental hits; also monitor the loop
*     _ counter as an extreme fall-back measure.
         do while
     *        (
     *        iexit.lt.10
     *        .and.
     *        ijtrm.lt.mktrm
     *        )
*     
*     Calculate the series.
            coef=dble(iktrm)*(dble(iktrm)+2.0d0)
            coef=2.0d0/coef
            trmf=coef*dcos((dble(iktrm)+1.0d0)*rxvar)
            acsfn=acsfn+trmf
            trmc=coef*dsin((dble(iktrm)+1.0d0)*rxvar)
            acsfc=acsfc+trmc
*     
*     Determine the larger of the two term variables.
            tmax=dabs(trmf)
            if (dabs(trmc).gt.tmax) then
               tmax=dabs(trmc)
            end if
*     
*     Calculate the error, using the size of the term.
            error=dabs(tmax)
*     
*     Check the error and update the exit flag.
            if (error.lt.ereff) then
               iexit=iexit+1
            else
               iexit=0
            end if
*     
*     Increment the counter.
            ijtrm=ijtrm+1
            iktrm=2*ijtrm+1
*     
         end do
*     
*     We should not have the maximum number of term be actually reached.
         if (ijtrm.ge.mktrm) then
            write(*,*) 'ERROR: maximum term count reached: ',ijtrm
            write(*,*) 'ERROR: aborting run; must update the program'
            stop
         end if
*     
*     Calculate the function.
         fnvar=xnorm*(1.0d0-acsfn)
*     
*     Calculate the conjugate.
         fcvar=xnorm*acsfc
*     
*     Write to stdout in filter mode.
         write(*,*) rxvar,fnvar,fcvar
*     
      end do
*     
      END
