*     JLdL 23Nov14.
*     
*     Add up the center series for the standard triangular wave until
*     _ one gets a given precision level, and count the terms.
*     
*     Note: this program is meant to be called from within the
*     _ shell script test_trnw_func_cent.test.
*     
*     Parameters:
*     _ mpnts: a fixed parameter for the variable npnts.
*     
*     Variables:
*     _ ermax: the maximum error acceptable in the result (input);
*     _ nspec: the number of special points showing in the graph;
*     _ onepi: the value of \pi;
*     _ snorm: the overall normalization constant of the series;
*     _ mktrm: the maximum number of terms to add up on the series;
*     _ npnts: the number of points in the x half interval;
*     _ delta: the increment of x within its interval;
*     _ trmsn: the graph-wise total number of series terms added;
*     _ mtrms: the single largest number of series terms added;
*     _ ixvar: the integer x variable (loop counter);
*     _ rxvar: the current real value of x;
*     _ xnorm: the x-dependent normalization constant;
*     _ exact: the exact value of the function at x;
*     _ acser: the accumulator for the series of the function;
*     _ iexit: the series loop exit flag;
*     _ ijtrm: the counter for the series terms (loop counter);
*     _ iktrm: the value of k for the series terms;
*     _ coef:  temporary for the coefficients of the series;
*     _ term:  temporary for a term of the series;
*     _ fnvar: the value of the function;
*     _ error: the current value of the error;
*     _ jtrms: the number of series terms added at the current point.
*     
      PROGRAM TEST_TRNW_FUNC_CENT
*     
*     Include global integer parameters;
*     _ this file defines mpnts.
      include 'global_params.f'
*     
*     Double precision everything.
      implicit double precision(a-h,o-z)
*     
*     Set the maximum acceptable error; read from stdin in filter mode.
      read(*,*) ermax
*     
*     Define the number of special points in graph; in this case there
*     _ are 2 special point(s), showing as 3 point(s) in the graph.
      nspec=3
*     
*     Define constants.
      onepi=dacos(-1.0d0)
*     
*     Define the global normalization for the series.
      snorm=4.0d0/onepi**2
*     
*     Set the maximum number of series terms to add up.
      mktrm=1000000
*     
*     Set the number of data points.
      npnts=mpnts
*     
*     Define the point spacing along x, and the range of x.
      delta=onepi/dble(npnts)
*     
*     Initialize the accumulator for the total numbers of terms.
      trmsn=0.0d0
*     
*     Initialize the storage for the maximum numbers of terms.
      mtrms=0
*     
*     Open an output file for the graph, for monitoring purposes.
      open(unit=1,file='graph.dat')
*     
*     Loop over the x interval.
      do ixvar=-npnts,npnts
*     
*     Avoid the special points.
         if ( ixvar.ne.-npnts
     *        .and.
     *        ixvar.ne.0
     *        .and.
     *        ixvar.ne.npnts ) then
*     
*     Define the current value of x.
            rxvar=dble(ixvar)*delta
*     
*     Define the x-dependent normalization.
            xnorm=snorm/dsin(rxvar)
*     
*     Define the exact value of the function.
            if (rxvar.gt.0.0d0) then
               exact=2.0d0*rxvar/onepi-1.0d0
            else
               exact=-2.0d0*rxvar/onepi-1.0d0
            end if
*     
*     Initialize the series accumulator for the function.
            acser=0.0d0
*     
*     Initialize the exit flag.
            iexit=0
*     
*     Initialize a loop counter for the terms.
            ijtrm=0
            iktrm=2*ijtrm+1
*     
*     Add up the series; loop until the value of the error decays below
*     _ the precision threshold; this has to remain so for 10 terms, in
*     _ order to filter out accidental hits; also monitor the loop
*     _ counter as an extreme fall-back measure.
            do while
     *           (
     *           iexit.lt.10
     *           .and.
     *           ijtrm.lt.mktrm
     *           )
*     
*     Calculate the series.
               coef=(dble(iktrm)*(dble(iktrm)+2.0d0))**2
               coef=4.0d0*(dble(iktrm)+1.0d0)/coef
               term=coef*dsin((dble(iktrm)+1.0d0)*rxvar)
               acser=acser+term
*     
*     Calculate the function.
               fnvar=-xnorm*acser
*     
*     Calculate the error.
               error=dabs(exact-fnvar)
*     
*     Check the error and update the exit flag.
               if (error.lt.ermax) then
                  iexit=iexit+1
               else
                  iexit=0
               end if
*     
*     Increment the counter.
               ijtrm=ijtrm+1
               iktrm=2*ijtrm+1
*     
            end do
*     
*     We should not have the maximum number of term be actually reached.
            if (ijtrm.ge.mktrm) then
               write(*,*) 'ERROR: maximum term count reached: ',ijtrm
               write(*,*) 'ERROR: aborting run; must update the program'
               stop
            end if
*     
*     Record the effective number of terms.
            jtrms=ijtrm-10
*     
*     Accumulate the total number of terms.
            trmsn=trmsn+dble(jtrms)
*     
*     Record the maximum number of terms.
            if (jtrms.ge.mtrms) then
               mtrms=jtrms
            end if
*     
*     Write the monitoring graphics data to the output file.
            write(1,*) rxvar,fnvar
*     
         end if
*     
      end do
*     
*     Write to stdout.
      write(*,*) 'maximum error: ',ermax
      write(*,*) 'average terms: ',trmsn/dble(2*npnts+1-nspec)
      write(*,*) 'maximum terms: ',dble(mtrms)
*     
      END
