*     JLdL 24Nov14.
*     
*     Add up the second-order center series for the Fourier conjugate
*     _ function of the first very slow-converging wave until one
*     _ gets a given high-precision level.
*     
*     Note: this program is meant to be called from within the
*     _ shell script tool_vs1w_conj_cent_2.tool.
*     
*     Parameters:
*     _ mpnts: a fixed parameter for the variable npnts.
*     
*     Variables:
*     _ ermax: the maximum error acceptable in the result (input);
*     _ nspec: the number of special points showing in the graph;
*     _ onepi: the value of \pi;
*     _ cnst1: a constant that appears in the final formulas;
*     _ cnst2: a constant that appears in the final formulas;
*     _ snorm: the overall normalization constant of the series;
*     _ mktrm: the maximum number of terms to add up on the series;
*     _ npnts: the number of points in the x half interval;
*     _ delta: the increment of x within its interval;
*     _ ixvar: the integer x variable (loop counter);
*     _ rxvar: the current real value of x;
*     _ xnorm: the x-dependent normalization constant;
*     _ ereff: the renormalized maximum acceptable error;
*     _ acser: the accumulator for the series of the function;
*     _ iexit: the series loop exit flag;
*     _ ijtrm: the counter for the series terms (loop counter);
*     _ iktrm: the value of k for the series terms;
*     _ coef:  temporary for the coefficients of the series;
*     _ term:  temporary for a term of the series;
*     _ fnvar: the value of the function;
*     _ error: the current value of the error.
*     
      PROGRAM TOOL_VS1W_CONJ_CENT_2
*     
*     Include global integer parameters;
*     _ this file defines mpnts.
      include 'global_params.f'
*     
*     Double precision everything.
      implicit double precision(a-h,o-z)
*     
*     Set the maximum acceptable error; read from stdin in filter mode.
      read(*,*) ermax
*     
*     Define the number of special points in graph; in this case there
*     _ are 1 special point(s), showing as 1 point(s) in the graph.
      nspec=1
*     
*     Define constants.
      onepi=dacos(-1.0d0)
      cnst1=2.0d0-dsqrt(2.0d0)
      cnst2=8.0d0-2.0d0*dsqrt(2.0d0)
*     
*     Define the global normalization for the series.
      snorm=1.0d0/(4.0d0*onepi)
*     
*     Set the maximum number of series terms to add up.
      mktrm=100000000
*     
*     Set the number of data points.
      npnts=mpnts
*     
*     Define the point spacing along x, and the range of x.
      delta=onepi/dble(npnts)
*     
*     Loop over the x interval.
      do ixvar=-npnts,npnts
*     
*     Avoid the special points.
         if ( ixvar.ne.0 ) then
*     
*     Define the current value of x.
            rxvar=dble(ixvar)*delta
*     
*     Define the x-dependent normalization.
            xnorm=snorm/(dsin(0.5d0*rxvar))**2
*     
*     Renormalize the maximum acceptable error accordingly.
            ereff=ermax/dabs(xnorm)
*     
*     Initialize the series accumulator for the function.
            acser=0.0d0
*     
*     Initialize the exit flag.
            iexit=0
*     
*     Initialize a loop counter for the terms.
            ijtrm=1
            iktrm=ijtrm
*     
*     Add up the series; loop until the value of the error decays below
*     _ the precision threshold; this has to remain so for 10 terms, in
*     _ order to filter out accidental hits; also monitor the loop
*     _ counter as an extreme fall-back measure.
            do while
     *           (
     *           iexit.lt.10
     *           .and.
     *           ijtrm.lt.mktrm
     *           )
*     
*     Calculate the series.
               coef=dble(iktrm)*(dble(iktrm)+1.0d0)*(dble(iktrm)+2.0d0)
               coef=(dsqrt( dble(iktrm)+0.0d0 )
     *              *      (dble(iktrm)+1.0d0)
     *              *      (dble(iktrm)+2.0d0)
     *              -2.0d0*(dble(iktrm)+0.0d0)
     *              *dsqrt( dble(iktrm)+1.0d0 )
     *              *      (dble(iktrm)+2.0d0)
     *              +1.0d0*(dble(iktrm)+0.0d0)
     *              *      (dble(iktrm)+1.0d0)
     *              *dsqrt( dble(iktrm)+2.0d0 )
     *              )*2.0d0/coef
               term=coef*dcos((dble(iktrm)+1.0d0)*rxvar)
               acser=acser+term
*     
*     Calculate the error, using the size of the term.
               error=dabs(term)
*     
*     Check the error and update the exit flag.
               if (error.lt.ereff) then
                  iexit=iexit+1
               else
                  iexit=0
               end if
*     
*     Increment the counter.
               ijtrm=ijtrm+1
               iktrm=ijtrm
*     
            end do
*     
*     We should not have the maximum number of term be actually reached.
            if (ijtrm.ge.mktrm) then
               write(*,*) 'ERROR: maximum term count reached: ',ijtrm
               write(*,*) 'ERROR: aborting run; must update the program'
               stop
            end if
*     
*     Calculate the conjugate.
            fnvar=xnorm*(cnst1-cnst2*(dsin(0.5d0*rxvar))**2-acser)
*     
*     Write to stdout in filter mode.
            write(*,*) ixvar,fnvar
*     
         end if
*     
      end do
*     
      END
